__author__ = 'tcaruso'


import math

from nhlstats.constants import FENWICK_EVENTS, SHOT_EVENTS, CORSI_EVENTS

NET_X_LOCATION = 89


def game_summary(gs):
    return {
        'date': gs['gameDate'][:10],
        'game_id': gs['gamePk'],
        'home_team': gs['teams']['home']['team']['name'],
        'home_score': gs['teams']['home']['score'],
        'away_team': gs['teams']['away']['team']['name'],
        'away_score': gs['teams']['away']['score'],
        'season': gs['season'],
        'game_state': gs['status']['detailedState'],
    }


def _calculate_distance(e):
    try:
        delta_x = NET_X_LOCATION - abs(int(e['x']))
        delta_y = -1 * int(e['y'])
        return math.sqrt(delta_x ** 2 + delta_y ** 2)
    except Exception as e:
        return None


def _calculate_angle(e):
    try:
        return abs(math.atan(e['y'] / (NET_X_LOCATION - abs(e['x']))) * (180 / math.pi))
    except:
        return None


def event(ev, max_players=1):
    event_type = ev['result']['eventTypeId']
    e = {
        'datetime': ev['about']['dateTime'],
        'period': ev['about']['period'],
        'period_time': ev['about']['periodTime'],
        'period_time_remaining': ev['about']['periodTimeRemaining'],
        'period_type': ev['about']['periodType'],
        'x': ev.get('coordinates', {}).get('x'),
        'y': ev.get('coordinates', {}).get('y'),
        'event_type': event_type,
        'event_secondary_type': ev['result'].get('secondaryType'),
        'event_description': ev['result'].get('description'),
        'is_shot': event_type in SHOT_EVENTS,
        'is_corsi': event_type in CORSI_EVENTS,
        'is_fenwick': event_type in FENWICK_EVENTS,
        'team_for': ev.get('team', {}).get('triCode')
    }

    e['shot_distance'] = _calculate_distance(e) if e['is_corsi'] else None
    e['shot_angle'] = _calculate_angle(e) if e['is_corsi'] else None

    players = ev.get('players', [])
    if len(players) < max_players:
        players.extend([{}] * (max_players - len(players)))

    for idx, player in enumerate(players):
        e['player_{}'.format(idx + 1)] = player.get('player', {}).get('fullName')
        e['player_{}_type'.format(idx + 1)] = player.get('playerType')
        e['player_{}_id'.format(idx + 1)] = player.get('player', {}).get('id')

    return e


def events(evs):
    max_players = max(len(e.get('players', [])) for e in evs)

    return [event(ev, max_players) for ev in evs]


def shift(sh):
    return {
        'id': sh['id'],
        'first_name': sh['firstName'],
        'last_name': sh['lastName'],
        'period': sh['period'],
        'shift_number': sh['shiftNumber'],
        'start_time': sh['startTime'],
        'end_time': sh['endTime'],
        'duration': sh['duration'],
        'team_abbreviation': sh['teamAbbrev'],
        'event_description': sh['eventDescription'],
        'event_details': sh['eventDetails'],
        'event_number': sh['eventNumber']
    }


__author__ = 'tcaruso'

import click

from datetime import date

from nhlstats.apiclient import list_games, list_plays, list_shots, list_shifts
from nhlstats.cli.helpers import ensure_yyymmdd_date
from nhlstats.cli.output import OutputFormat


@click.group()
def cli():
    pass


@cli.command(name='list-games')
@click.argument('start_date', default=str(date.today()), callback=ensure_yyymmdd_date)
@click.argument('end_date', default=str(date.today()), callback=ensure_yyymmdd_date)
@click.option('--output-format', type=click.Choice(OutputFormat.options()), default='text', callback=OutputFormat.from_click_option)
def _list_games(start_date, end_date, output_format: OutputFormat):
    """
    List all games from START_DATE to END_DATE. Dates should be of the form YYYY-MM-DD. Both date arguments default to
    "today" by system time, so you can omit the final argument to get a range from the first date to today.
    """
    output_format.echo(list_games(start_date, end_date))


@cli.command(name='list-plays')
@click.argument('game_id')
@click.option('--output-format', type=click.Choice(OutputFormat.options()), default='text', callback=OutputFormat.from_click_option)
def _list_plays(game_id, output_format: OutputFormat):
    """
    List all play events which occurred in the given GAME_ID.
    """
    output_format.echo(list_plays(game_id))


@cli.command(name='list-shots')
@click.argument('game_id')
@click.option('--output-format', type=click.Choice(OutputFormat.options()), default='text', callback=OutputFormat.from_click_option)
def _list_shots(game_id, output_format: OutputFormat):
    """
    List all shot events which occurred in the given GAME_ID.
    """
    output_format.echo(list_shots(game_id))


@cli.command(name='list-shifts')
@click.argument('game_id')
@click.option('--output-format', type=click.Choice(OutputFormat.options()), default='text', callback=OutputFormat.from_click_option)
def _list_shifts(game_id, output_format: OutputFormat):
    """
    List all shifts which occurred in the given GAME_ID.
    """
    output_format.echo(list_shifts(game_id))





__author__ = 'tcaruso'

import importlib

import click

from enum import Enum


class OutputFormat(Enum):
    TEXT = 'text'
    CSV = 'csv'
    JSON = 'json'

    @staticmethod
    def options():
        return list(out.value for out in OutputFormat)

    @classmethod
    def from_click_option(cls, ctx, param, value):
        try:
            return cls(value)
        except:
            raise click.BadParameter(value)

    def echo(self, events: list) -> None:
        try:
            formatter = importlib.import_module('nhlstats.formatters.{}'.format(self.value))
            click.echo(formatter.dumps(events))
        except ImportError:
            raise click.UsageError("Output format {} is not implemented.".format(self.value))
        except Exception as e:
            raise click.Abort(str(e))



__author__ = 'tcaruso'

import re

import click


def ensure_yyymmdd_date(ctx, param, value):
    match = re.search(r'\d\d\d\d-\d\d-\d\d', value)

    if match is None:
        raise click.BadParameter("Date must be of the form YYYY-MM-DD.")

    return value




__author__ = 'tcaruso'

from itertools import chain as _chain


def _sanitize(value):
    return str(value) if value is not None else ''


def _create_line(values, separator):
    return separator.join(_sanitize(val) for val in values)


def dump(plays, fobj, seperator=','):
    fobj.write(dumps(plays, seperator=seperator))


def dumps(plays, seperator=','):
    header = seperator.join(plays[0].keys())
    rows = (_create_line(play.values(), seperator) for play in plays)

    return '\n'.join(_chain([header], rows))
